#!/bin/sh

metrics_user_exists() {
  [ $(drone -s "$DRONE_DOMAIN" -t "$PRIMARY_DRONE_USER_TOKEN" user ls 2>/dev/null | grep -c '^build-metrics$') -ne 0 ]
  # `return` on its own will return the return code of the previous statement.
  # Bash is a very sensible and normal programming language
  return
}

if metrics_user_exists; then
  echo "Drone Build Metrics user exists - exiting"
  exit 0
else
  # I'm assuming that it needs to be admin in order to see and report on every build,
  # but if you wanted to get really finicky with permissions you could create different
  # metrics users with different permissions.
  drone -s "$DRONE_DOMAIN" -t "$PRIMARY_DRONE_USER_TOKEN" \
    user add --machine --admin --token "$METRICS_DRONE_USER_TOKEN" build-metrics
  # Double-check!
  if ! metrics_user_exists; then
    echo "Tried creating the user, but it still doesn't exist - something's gone wrong"
    echo "Drone Domain: $DRONE_DOMAIN"
    echo "Primary Drone User Token: $PRIMARY_DRONE_USER_TOKEN"
    exit 1
  fi
  exit 0
fi
